/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "umtsmon_global.h"
#include "ConnectionInfo.h"
#include "Query.h"

int ConnectionInfo::theInhibitionCount = 0;

bool ConnectionInfo::isInhibited(void)
{
	if (theInhibitionCount>0)
	{
		DEBUG3("ConnectionInfo probe inhibited\n");
		return true;
	}
	return false;
}


void ConnectionInfo::refresh(void)
{
	// only send Query if allowed
	if (isInhibited())
		return;

	refreshSignalQuality();
	
	theRegistrationValue = refreshCGREG();
	// if not successful, retry with AT+CREG
	if (theRegistrationValue == REGISTRATION_UNKNOWN)
		theRegistrationValue = refreshCREG();

	if (isDeviceRegistered())
		refreshOperator();
	else
	{
		if (isSearching())
			theOperatorName = "Searching...";
		else	
			theOperatorName = "n/a";
	}
}



void ConnectionInfo::refreshSignalQuality(void)
{
	// only send Query if allowed
	if (isInhibited())
		return;
	
	// only do this when registered 
	//  - data won't make sense otherwise	
	if (!isDeviceRegistered())
		theSignalQuality=99;
	else
	{
		Query myQuery("AT+CSQ");
		if (myQuery.runUntilDone() != Query::OK)
		{
			// no big deal, it happens, fail silently
			DEBUG2("Query.run() failed\n");
			return;
		}
		
		// result: '+CQS: 19,0'  -> '19,0'
		QString myResult = myQuery.getAnswer().first().section(" ",1,1);
		// result: '19,0' -> int 19 and display it...
		theSignalQuality = myResult.section(",",0,0).toInt();
	}
}


void ConnectionInfo::refreshOperator(void)
{
	// only send Query if allowed
	if (isInhibited())
		return;

	// query for current operator
	Query myQuery("AT+COPS?");
	if (myQuery.runUntilDone() != Query::OK)
		{
			// no big deal, it happens, fail silently
			DEBUG4("Query.run() failed...\n");
			return;
		}
	QString myAnswer = myQuery.getAnswer().first();
	if (myAnswer == "+COPS: 0")
	{
		theOperatorName = "n/a";
		theRadioType  = -1;
	}
	else
	{
		// result: '+COPS: 0,0,"vodafone SE",0'  -> 'vodafone SE'
		theOperatorName = myAnswer.section("\"",1,1);
		// filter after the second quote+comma -> ,2
		theRadioType = myAnswer.section("\",",1,1).toInt();
	}
}

QString 
ConnectionInfo::getRadiotypeString(int aRadioType)
{
	QString theType;
	switch (aRadioType)
	{
	case -1:
		theType = "n/a";
		break;
	case 0:
		// officially: GSM
		theType = "GPRS";
		break;
	case 1:
		// officially: GSM compact
		theType = "GSM compact";
		break;
	case 2:
		// officially: UTRAN
		theType = "UMTS";
		break;
	default:
		theType = "???";
		break;
	}

	return theType;
}


QString ConnectionInfo::getRadiotype(void)
{ 
	if ( theRegistrationValue==REGISTERED_HOME ||
	     theRegistrationValue==REGISTERED_ROAMING)
		return ConnectionInfo::getRadiotypeString(theRadioType);
	else
	{
		if (theRegistrationValue==TRYING_TO_REGISTER)
			return "Searching...";
		else
			return "n/a";
	} 
}

// this member contains the implementation of the official specification
// 3GPP TS 27.007 V7.0.0 (2006-03) - chapter 10.1.19
// unfortunately, this command is only 'Optional'.
ConnectionInfo::Registration ConnectionInfo::refreshCGREG(void)
{
	Query myQuery( "AT+CGREG?" );
 	Query::ReturnCode myResult = myQuery.runUntilDone();
 	
 	if (myResult != Query::OK)
 	{
 		DEBUG2("unfortunately REGISTRATION_UNKNOWN as answer to AT+CGREG?\n");
 		return REGISTRATION_UNKNOWN;
 	}
	
	// answer should be something like: +CGREG: <n>,<stat>[,<lac>,<ci>]
	// where we don't care about <n>
	// and <stat> can be 0,1,2,3,4,5 
	// For now, we don't care about <lac> or <ci> either
	QString myAnswer = myQuery.getAnswer().first();
	if (!myAnswer.contains("CGREG"))
 	{
 		DEBUG2("unfortunately unparsible answer '%s' to AT+CGREG.\n", myAnswer.ascii());
 		return REGISTRATION_UNKNOWN;
 	}
	
	bool isOK;
	int myCGREG = myAnswer.section(",",1,1).toInt(&isOK);
	
	if (isOK)
		return static_cast<Registration>(myCGREG);
	else
		return REGISTRATION_UNKNOWN;
}

// this member contains the implementation of the official specification
// 3GPP TS 27.007 V7.0.0 (2006-03) - chapter 7.2
// unfortunately, this command is only 'Optional'.
ConnectionInfo::Registration ConnectionInfo::refreshCREG(void)
{
	Query myQuery( "AT+CREG?" );
 	Query::ReturnCode myResult = myQuery.runUntilDone();
 	
 	if (myResult != Query::OK)
 	{
 		DEBUG2("unfortunately REGISTRATION_UNKNOWN as answer to AT+CREG?\n");
 		return REGISTRATION_UNKNOWN;
 	}
	
	// answer should be something like: +CREG: <n>,<stat>[,<lac>,<ci>]
	// where we don't care about <n>
	// and <stat> can be 0,1,2,3,4,5 
	// For now, we don't care about <lac> or <ci> either
	QString myAnswer = myQuery.getAnswer().first();
	if (!myAnswer.contains("CREG"))
 	{
 		DEBUG2("unfortunately unparsible answer '%s' to AT+CREG.\n", myAnswer.ascii());
 		return REGISTRATION_UNKNOWN;
 	}
	
	bool isOK;
	int myCGREG = myAnswer.section(",",1,1).toInt(&isOK);
	
	if (isOK)
		return static_cast<Registration>(myCGREG);
	else
		return REGISTRATION_UNKNOWN;
}
