/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "umtsmon_global.h"
#include "DetectCard.h"
#include "Device.h"
#include "FileStuff.h"
#include "Popup.h"

#include <assert.h>
#include <qdir.h>
#include <qmessagebox.h>


bool
CardDetection::detectPorts(void)
{ 
	VendorIDs myVendor = UNKNOWN;
	theVendorID = UNKNOWN;
	theDeviceID = Device::UNKNOWN;
	
	// if there already is a serial port name set, no need to autodetect
	// this is a programming error now - choose autodetection or pre-configuring
	// not both
	assert(theATSerialPortName.isEmpty());
	assert(thePPPSerialPortName.isEmpty());
	
	// if not a 2.6 kernel, umtsmon's autodetect cannot work...
	if (!is26kernel())
	{
		sendLog(INFO, "No Linux 2.6 kernel found.");
		sendLog(KILLING, "This means autodetection cannot be performed, please supply info on commandline"); 
		return false;
	}

	myVendor = doUSBDetection();
	if (myVendor == UNKNOWN)
	{
		if (isPCMCIA())
		{
			myVendor = doPCMCIA_PCIDetection();
			if (myVendor == UNKNOWN)
				myVendor = doCSSerialDetection();
		}
	}
	if (myVendor == UNKNOWN)
		myVendor = doPhoneModemDetection();
	if (myVendor == UNKNOWN)
		myVendor = doRawUSBSerialDetection();
		
	// * if we are still unknown - let's check for regular usb2serial
	// * for all known usb2serial based devices, figure out the usb2serial ports
	// * for all nozomi based devices, figure out the nozomi ports	
	switch (theDeviceID)
	{
	case Device::OPTION_NOZOMI:
	    assert(isNozomi());
		break;
	case Device::NOVATEL_2PORT:
		// 2 ports: /dev/ttyUSB0 and /dev/ttyUSB1
		// unfortunately, we can only use the first port:
		// so let's continue below and fix it up afterwards
	case Device::OPTION_VODAFONE_3G:
	case Device::OPTION_3PORT:
	case Device::HUAWEI_E612:
		if  (!isUSBSerial())
		{
			sendLog(BAD, "usb serial ports expected, but none found");
			sendLog(BAD, "maybe you need to run modprobe manually:");
			char buffer[256];
			// FIXME: Add the deviceID here - theDeviceID is wrong
			// FIXME: this is the wrong location
			// FIXME: Add a check on the usbserial and/or option kernel modules
			snprintf(buffer, 255, "modprobe usbserial vendor=0x%x product=0x%x",
						theVendorID, theDeviceID);
			sendLog(BAD, buffer);
			return false;
		}
		if (theDeviceID==Device::NOVATEL_2PORT)
		{
			// as stated above - xu870 has only one port
			theATSerialPortName = thePPPSerialPortName;
		}
		break;
	default:
		break;
	}	
	
	
	if (myVendor == UNKNOWN)
		return false;
	else
		return true;
}

bool CardDetection::createDevice()
{	
	assert(theDevicePtr == NULL);
	// FIXME: there are several assert(false) in this member that must be fixed
	switch (theVendorID)
	{
		case KYOCERA:
		case NOT_SUPPORTED:
			Popup::Critical("This card unfortunately is not supported by " APPNAME "\n"
						"We are very sorry, please contact the developers on how "
						"to work on this.");
			return false; 
		case PSITEK:
			theDevicePtr =Device::createDeviceInstance(theDeviceID);
			break;

		case OPTI:
			sendLog(KILLING, "VendorID OPTI: this shouldn't happen");
			return false;
		
		case FOURGSYSTEMS_USB:
			sendLog(INFO, "FIXME: hopefully /dev/ttyACM0 is OK");
			theATSerialPortName = "/dev/ttyACM0";
			theDevicePtr = Device::createDeviceInstance(Device::GENERIC_USER_SERIAL);
			break;
			
		case OPTION_USBSTORAGE:
		case FOURGSYSTEMS_USBSTORAGE:
			Popup::Critical("your UMTS/HSDPA device is still in storage mode.\n"
						"Please use icon_switch to switch it to modem mode."); 
			sendLog(KILLING, "please use 'icon_switch' to switch your box to modem mode first");
			return false;
		
		case NOVATEL:
		case UNNAMED_MULTI_SERIAL:
		case OPTION_USB:
		case OPTION:
			// FIXME: this is in the wrong spot
			if (theDeviceID==Device::OPTION_GTEDGE)
			{
				// FIXME: hard coded serial port for now:
				theATSerialPortName = "/dev/ttyS4";
				break;
			}
			theDevicePtr=Device::createDeviceInstance(theDeviceID);
			break;
		case BROADCOM:
			// FIXME: hard coded serial port for now:
			theATSerialPortName = "/dev/ttyS14";
			theDevicePtr =Device::createDeviceInstance(Device::BROADCOM_GPRS_EDGE);
			break;
		case UNNAMED_SINGLE_SERIAL:
		case SIERRAWIRELESS:
			// these are all easy devices 
			theDevicePtr = Device::createDeviceInstance(theDeviceID);
			break;
		case NEC:
			// NEC shouldn't appear by itself, we only know about Option here
			// so, let's skip on to UNKNOWN
		case UNKNOWN:
			// intentional missing 'break': the unknown case continues in user-specified 
		case USER_SPECIFIED:
			theDevicePtr = Device::createDeviceInstance(Device::GENERIC_USER_SERIAL);
			break;
	}

	assert(theDevicePtr != NULL);
	
	// setupATPort already performs isCardResponding
	if (!theDevicePtr->setupATPort(theATSerialPortName))
	{
		sendLog(KILLING, "Device not responding on " + theATSerialPortName);
		return false;
	}
	if (!theDevicePtr->setupPPPPort(thePPPSerialPortName))
	{
		sendLog(KILLING, "Device not responding on " + thePPPSerialPortName);
		return false;
	}

// FIXME: THIS IS THE WRONG LOCATION HERE
	if (theDevicePtr->hasRadioPreferences())
	{
		// FIXME: Do not care if this fails at this point, as it might be
		// totally inappropriate to try set such connection at this point
		// (e.g. if a PPP connection is already made outside umtsmon)
		theDevicePtr->setRadioPreferences(theDevicePtr->getDefaultRadioPreference());
	}

	return true;
}




void
CardDetection::setSerialPortNames(const QString& anATSerialPortName, const QString& aPPPSerialPortName)
{
	sendLog(INFO, "Command line specified GENERIC_USER_SERIAL device"); 
	theATSerialPortName = anATSerialPortName;
	thePPPSerialPortName = aPPPSerialPortName;
	theVendorID  =  USER_SPECIFIED;
	theDeviceID  =  Device::GENERIC_USER_SERIAL;
}


//////////////////////////////////////////////////////////////////////////////
bool 
CardDetection::is26kernel(void)
{ 
	if ( ! FileStuff::doesFileContainString("/proc/version", QString("2.6.\\d\\d?")) )
	{
		sendLog(KILLING, "no 2.6 kernel found");
		return false;
	}
	sendLog(INFO, "2.6 kernel found");

	if ( ! FileStuff::doesFileExist("/sys") )
	{
		sendLog(KILLING, "no sysfs mounted on /sys");
		return false;
	}
	sendLog(INFO, "sysfs found, mounted on /sys");

	return true;
}



//////////////////////////////////////////////////////////////////////////////
bool 
CardDetection::isPCMCIA(void)
{ 
	int numberOfOKs = 0;
	// check for pcmcia_core in /proc/modules (=lsmod)
	if (FileStuff::doesFileContainString("/proc/modules", QString("pcmcia_core")))
	{
		numberOfOKs++;
		sendLog(INFO, "pcmcia_core kernel support (yenta) found");
	}
	else
	{
		sendLog(BAD, "pcmcia_core kernel support (yenta) not found");
	}
	if (FileStuff::doesFileContainString("/proc/devices", QString("pcmcia")))
	{
		numberOfOKs++;
		sendLog(INFO, "PCMCIA kernel module found - cardctl support possible");
	}
	else
	{	sendLog(BAD, "PCMCIA kernel module not found - you won't have cardctl support");
	}

	// check for pcmcia in /sys/class
	thePCMCIASocketsDirectory = "/sys/class/pcmcia_socket";
	if (FileStuff::doesFileExist(thePCMCIASocketsDirectory))
	{
		numberOfOKs++;
		sendLog(INFO, "pcmcia browsing available");
	}

	if (numberOfOKs > 0)
		return true;
	else
		return false;
}



//////////////////////////////////////////////////////////////////////////////
CardDetection::VendorIDs 
CardDetection::doRawUSBSerialDetection(void)
{
	// let's detect if there's a USBSerial (first on PCMCIA...)
	sendLog(INFO, "Nothing detected yet - let's check for usb2serial in general");
	if (!isUSBSerial(true))
	{
		sendLog(BAD, "no usbserial ports found that are linked to PCMCIA");
		
		// any ports without a link to pcmcia?
		if (!isUSBSerial(false))
		{
			sendLog(KILLING, "no USB2Serial found, I'm out of options now...");
			return UNKNOWN;
		}
	}
	if (theATSerialPortName == thePPPSerialPortName)
	{
		sendLog(GOOD, "setting up unnamed single serial device");
		theVendorID = UNNAMED_SINGLE_SERIAL;
		theDeviceID = Device::GENERIC_USER_SERIAL;
	}
	else
	{
		sendLog(GOOD, "setting up unnamed multiport serial device");
		theVendorID = UNNAMED_MULTI_SERIAL;
		theDeviceID = Device::OPTION_3PORT;
	}
	return theVendorID;
}



//////////////////////////////////////////////////////////////////////////////
CardDetection::VendorIDs 
CardDetection::doPhoneModemDetection(void)
{
	// This is rather course detection - we don't even bother to check
	// if the devices are actually present and operational and such
	
	QString mySerial = "/dev/ttyACM0";
	if (FileStuff::doesFileExist(mySerial))
	{
		sendLog(GOOD, "USB-connected phone modem detected"); 
		theATSerialPortName = mySerial;
		theVendorID  =  UNNAMED_SINGLE_SERIAL;
		theDeviceID  =  Device::MODEM_ACM;
		return theVendorID;
	}
	
	mySerial = "/dev/rfcomm0"; 
	if (FileStuff::doesFileExist(mySerial))
	{
		sendLog(GOOD, "Bluetooth based phone modem detected"); 
		theATSerialPortName = mySerial;
		theVendorID  =  UNNAMED_SINGLE_SERIAL;
		theDeviceID  =  Device::MODEM_RFCOMM;
		return theVendorID;
	}
	
	sendLog(INFO, "No Phone Modem found");
	return UNKNOWN;
}


//////////////////////////////////////////////////////////////////////////////
CardDetection::VendorIDs 
CardDetection::doCSSerialDetection(void)
{
	// check if the cardmgr process is running
	// on heavily tweaked systems, a user might have setup serial_cs himself
	// so, this is not critical
	if (FileStuff::checkForRunningProcess("cardmgr"))
	{
		sendLog(INFO, "The PCMCIA card manager process (\"cardmgr\") is running.");
	}
	
	// check if the serial_cs module is loaded
	if (!FileStuff::doesFileExist("/sys/bus/pcmcia/drivers/serial_cs"))
	{
		sendLog(INFO, "no serial_cs capable device found");
		return UNKNOWN;
	}
	sendLog(GOOD, "a PCMCIA serial modem found");
	

	// link the serial port to this device
	// this apparently is quite complicated, as the serial port is just seen
	// as a regular ISA interrupt with a regular serial device on it
	
	// According to this website: http://pcmcia-cs.sourceforge.net/
	// there are two different PCMCIA sub systems in use, the pre-2.6.13-rc1 one,
	// called pcmcia-cs   and the post-2.6.13-rc1 one, which does a lot of things better
	
	// apparently, in higher kernels than 2.6.13-rc1, there is a direct link to the serial port:
	// according to Falk Willenberg:
	// 
	// "/sys/class/pcmcia_socket/pcmcia_socket0/device/0.0/tty:ttyS0", 
	// (which is a link to "../../../../../class/tty/ttyS0", 
	// which is a link to .../ and so on) and tries to install as a driver for a /dev/ttyS?
	//
	// However, as I don't run such kernels (yet), I cannot check.
	// so, for the lower kernels, let's just do it the hard way:

	// however, as a pre-2.6.13-rc1 user, you're left to your own.
	// this is getting too complicated to fix now :-(

	// let's just be *very* blunt and probe each port until we find one that responds to AT commands...
	sendLog(INFO, "No clean detection of serial ports for serial PCMCIA modems yet, probing first ten ports");
	bool isSuccess = false;
	Device* myDevicePtr = Device::createDeviceInstance(Device::GENERIC_CS_SERIAL);
	int myPortNr;
	for (myPortNr=0; myPortNr<10; myPortNr++)
	{
		theATSerialPortName = "/dev/ttyS" + theATSerialPortName.setNum(myPortNr);
		if (myDevicePtr->setupATPort(theATSerialPortName))
		{
			QString myMessage = "Found a modem on port " + theATSerialPortName; 
			sendLog(GOOD, myMessage);
			isSuccess = true;
			break;
		}
		QString myMessage = "No responding modem on port " + theATSerialPortName; 
		sendLog(BAD, myMessage);
		myDevicePtr->closeATSerial();
	}
	delete myDevicePtr;
	
	// let's check if /dev/modem points somewhere
	if (!isSuccess)
	{
		QString mySerial = "/dev/modem"; 
		if (FileStuff::doesFileExist(mySerial))
		{
			sendLog(INFO, "/dev/modem exists");
			Device* myDevicePtr = Device::createDeviceInstance(Device::GENERIC_CS_SERIAL);
			if (myDevicePtr->setupATPort(mySerial))
			{
				theATSerialPortName = mySerial;
				QString myMessage = "Found a modem on port " + theATSerialPortName; 
				sendLog(GOOD, myMessage);
				isSuccess = true;
			}
			myDevicePtr->closeATSerial();
			delete myDevicePtr;
		}
	}
	
	if (isSuccess)
	{
		thePPPSerialPortName = theATSerialPortName;
	
	// FIXME: I'm just going to do a hardcoded Sierra Wireless Aircard 775 here.
	// I'm sorry Novatel, I don't have your cards yet.
	// For future reference, the info from the Sierra Wireless: 
	//		klaas@jip:/> /sbin/cardctl info
	//		PRODID_1="Sierra Wireless"
	//		PRODID_2="AC775"
	//		PRODID_3="EDGE Network Adapter"
	//		PRODID_4="R1"
	//		MANFID=0192,0710
	//		FUNCID=2
	//		klaas@jip:/> /sbin/cardctl ident
	//		Socket 0:
	//		  product info: "Sierra Wireless", "AC775", "EDGE Network Adapter", "R1"
	//		  manfid: 0x0192, 0x0710
	//		  function: 2 (serial)
	//
	// Keep in mind that the Sierra Wireless cards only work if the correct firmware is 
	// present on the system AND loaded by udev (2.6.13+)
	// (copy the *.cis files from the SierraWireless website to /lib/firmware)
	// (google keywords: Sierra Wireless ac710 Linux)
	// otherwise, you won't see any serial ports
	// FIXME: Add checks to find out if the firmware is loaded
		theVendorID=SIERRAWIRELESS;
		theDeviceID=Device::GENERIC_CS_SERIAL;
		return SIERRAWIRELESS;
	}
	else
	{
		theATSerialPortName = "";
		return UNKNOWN;
	}
}


//////////////////////////////////////////////////////////////////////////////
// this member iterates through the PCMCIA socket
// however, that list only contains the PCI devices
// if the device is a ISA device (like serial_cs cards are), it won't be
// available in this list.
CardDetection::VendorIDs
CardDetection::doPCMCIA_PCIDetection(void)
{
	// work from thePCMCIASocketsDirectory here
	// it will contain at least one and maybe multiple pcmcia_socket dirs
    // each containing a link to a device
	QDir myPCMCIASockDir(thePCMCIASocketsDirectory);
	myPCMCIASockDir.setFilter(QDir::Dirs);
	myPCMCIASockDir.setNameFilter("pcmcia_socket*");
	const QFileInfoList* mySocketsList = myPCMCIASockDir.entryInfoList();
	QFileInfoListIterator mySockListEntry(*mySocketsList);
	QFileInfo* mySockDirInfo;
	while ( (mySockDirInfo=mySockListEntry.current()) != 0 )
	{
		QString myPCMCIASocketDevice = thePCMCIASocketsDirectory + "/" + mySockDirInfo->fileName()+ "/device";
		QFileInfo mySockDev(myPCMCIASocketDevice);
		if (!mySockDev.isSymLink())
		{
			++mySockListEntry;
			continue;
		}
		thePCMCIASocketDeviceLink = mySockDev.readLink();

		// the directory contains several entries, usually starting with 0000. 
		// So let's iterate over those entries and check for Vendor IDs
		QDir myPCMCIADir(myPCMCIASocketDevice);
		myPCMCIADir.setFilter( QDir::Dirs );
		myPCMCIADir.setNameFilter ( "0000:*" );

		const QFileInfoList* myList = myPCMCIADir.entryInfoList();
		QFileInfoListIterator myIterator( *myList );
		
		for (QFileInfo *myFileInfo; 
			 (myFileInfo=myIterator.current()) != 0; ++myIterator)
		{
			QString myVendorID;
			QString myDeviceID;
			QString mySubVendorID;
			
			QString myDirName = myFileInfo->fileName();
		
			QString myTemp = myPCMCIASocketDevice + "/" + myDirName + "/vendor";
			if (!FileStuff::doesFileExist(myTemp))
				sendLog(BAD, "Could not open file: " + myTemp);
			else
				myVendorID = FileStuff::getFileFirstLine(myTemp);
			
			myTemp = myPCMCIASocketDevice + "/" + myDirName + "/device";
			if (!FileStuff::doesFileExist(myTemp))
				sendLog(BAD, "Could not open file: " + myTemp);
			else
				myDeviceID = FileStuff::getFileFirstLine(myTemp);
			
			myTemp = myPCMCIASocketDevice + "/" + myDirName + "/subsystem_vendor";
			if (!FileStuff::doesFileExist(myTemp))
				sendLog(BAD, "Could not open file: " + myTemp);
			else
				mySubVendorID = FileStuff::getFileFirstLine(myTemp);

			switch(myVendorID.toULong(NULL,0))
			{
			case OPTION:
				sendLog(GOOD, "Detected Option Nozomi based card \"Nozomi\" PCMCIA card");
				theVendorID = OPTION;
				theDeviceID = Device::OPTION_NOZOMI;
				return OPTION;
			case BROADCOM:
				sendLog(INFO, "Detected BroadCom-based PCMCIA card");
				sendLog(INFO, "Broadcom device ID: " + myDeviceID);
				
				if (myDeviceID.toULong(NULL,0) == 0x4322)
				{
					sendLog(GOOD, "Detected BroadCom-based PCMCIA card with 4306 UART, probably Sony Ericsson GC79");
					theVendorID = BROADCOM;
					theDeviceID = Device::BROADCOM_GPRS_EDGE;
					return BROADCOM;
				}
				else if (myDeviceID.toULong(0,0) == 0x4333)
				{
					sendLog(GOOD, "Detected BroadCom-based PCMCIA card Option GT EDGE");
					theVendorID = OPTION;
					theDeviceID = Device::OPTION_GTEDGE;
					return OPTION;
				}
				sendLog(BAD, "Unknown BroadCom device detected - might be just wifi card\n");	
				break;
			case NEC:
				sendLog(INFO, "Detected NEC-based usb2serial PCMCIA card");
				switch(mySubVendorID.toULong(NULL,0))
				{
				case OPTION:
					sendLog(GOOD, "Detected \"Option 3G GT\" PCMCIA card");
					theVendorID = OPTION;
					theDeviceID = Device::OPTION_3PORT;
					return OPTION;
				case PSITEK:
					sendLog(GOOD, "Detected \"Huawei E612\" PCMCIA card");
					theVendorID = PSITEK;
					theDeviceID = Device::HUAWEI_E612;
					return PSITEK;
				case KYOCERA:
					sendLog(INFO, "Detected \"Kyocera KPC650\" PCMCIA card");
					sendLog(BAD, "This card is not supported !!!");
					theVendorID = NOT_SUPPORTED;
					theDeviceID = Device::OPTION_4PORT;
					return NOT_SUPPORTED;
				case NEC:
					// we encountered this one with Matthias Lemke's Novatel XU870 in adapter.
					// for some reason, the sub IDs are equal to the main IDs.
					sendLog(INFO, "subvendor equals vendor detected. Is there an adapter around your card?");
					sendLog(INFO, "Anyway, we think we can handle this one by doing direct USB analysis.");
					// so we don't set anything and return - let the USB detection find it 
					return UNKNOWN;
				default:
					sendLog(BAD, "No known NEC-based card");
					theVendorID = NOT_SUPPORTED;
					theDeviceID = Device::OPTION_4PORT;
					return OPTION;
				}
				break;
			case OPTI:
				// for future reference, in case we find more 'original 3G' cards,
				// we are talking about:
				//    VendorID:    0x0x1045
				//    DeviceID:    0x0xc861
				//    SubVendorID: 0x0x1045
				sendLog(GOOD, "Detected Original Vodafone 3G card");
				theVendorID = OPTION;
				theDeviceID = Device::OPTION_VODAFONE_3G;
				return OPTION;
			default:
				myTemp = myPCMCIASocketDevice + "/" + myDirName;
				DEBUG5("PCMCIA card found, but not detected: '%s'\n", myTemp.ascii());
				DEBUG5("VendorID:    0x%4s\n", myVendorID.ascii());
				DEBUG5("DeviceID:    0x%4s\n", myDeviceID.ascii());
				DEBUG5("SubVendorID: 0x%4s\n", mySubVendorID.ascii());
				break;
			} // end-of-switch
			
		} // end-of-for
		++mySockListEntry;
	} // end-of-while
	sendLog(BAD, "no PCMCIA device found");
	return UNKNOWN;
}


//////////////////////////////////////////////////////////////////////////////
// this member iterates through sysfs USB directory 
CardDetection::VendorIDs 
CardDetection::doUSBDetection(void)
{
	// FIME: Not Implemented yet
	
	
		// the below directory contains several entries 
		// all USB devices are directly linked and start with a digit
		// so we can safely ignore the others
		// al subdirs should have an idVendor file containing the vendor ID
	QString myUSBDeviceDir = "/sys/bus/usb/devices"; 
	QDir myUSBDir(myUSBDeviceDir);
	myUSBDir.setFilter( QDir::Dirs );
	myUSBDir.setNameFilter ( "[0-9]*" );

	const QFileInfoList* myList = myUSBDir.entryInfoList();
	QFileInfoListIterator myIterator( *myList );
	
	for (QFileInfo *myFileInfo; 
		 (myFileInfo=myIterator.current()) != 0; ++myIterator)
	{
		unsigned long myVendorID;
		unsigned long myProductID;
		QString myDirName = myFileInfo->fileName();
	
		QString myTemp = myUSBDeviceDir + "/" + myDirName + "/idVendor";
		if (FileStuff::doesFileExist(myTemp))
			myVendorID = QString("0x" + FileStuff::getFileFirstLine(myTemp)).toULong(NULL,0);
		else
			continue; // better luck next while iteration

		myTemp = myUSBDeviceDir + "/" + myDirName + "/idProduct";
		if (FileStuff::doesFileExist(myTemp))
			myProductID = QString("0x" + FileStuff::getFileFirstLine(myTemp)).toULong(NULL,0);
		else
			continue; // better luck next while iteration

		DEBUG5("USB iteration: vendorID/ProductID = 0x%04lx:%04lx\n", 
				 myVendorID, myProductID);

		// If this switch() grows another 3 entries (i.e. 7+ entries), 
		// it is probably smarter to implement this iterating through an array.
		switch(myVendorID)
		{
		case NOVATEL:
			if (myProductID == 0x1430)
			{
				sendLog(GOOD, "Detected Novatel XU870 HSDPA modem"); 
				theVendorID = NOVATEL;
				theDeviceID = Device::NOVATEL_2PORT;
				return NOVATEL;
			}
			break;
		case OPTION_USBSTORAGE:
			if (myProductID == 0x1000)
			{
				sendLog(GOOD, "Detected Option Icon or similar - in storage mode\n"); 
				theVendorID = OPTION_USBSTORAGE;
				theDeviceID = Device::OPTION_4PORT;
				return OPTION_USBSTORAGE;
			}
			break;
		case OPTION_USB:
			sendLog(GOOD, "Found Option USB device");
			theVendorID = OPTION;
			theDeviceID = Device::OPTION_3PORT;
			return OPTION;
		case FOURGSYSTEMS_USBSTORAGE:
			if (myProductID == 0x4e44)
			{
				sendLog(GOOD, "Detected 4GSystems box or similar - in storage mode\n"); 
				theVendorID = FOURGSYSTEMS_USBSTORAGE;
				theDeviceID = Device::UNKNOWN;
				return FOURGSYSTEMS_USBSTORAGE;
			}
			break;
		case FOURGSYSTEMS_USB:
			theVendorID = FOURGSYSTEMS_USB;
			// FIXME: Do we need a new Device class here???
			theDeviceID = Device::GENERIC_USER_SERIAL;
			return UNKNOWN;
			break;			 
		default:
			// Well, if the info is available anyway, let's list it here:
			myTemp = myUSBDeviceDir + "/" + myDirName;
			DEBUG5("  unknown USB Device '%s' '%s'\n",
					FileStuff::getFileFirstLine(myTemp+"/manufacturer").ascii(),
					FileStuff::getFileFirstLine(myTemp+"/product").ascii());
					
			break;
		}
	} 

	sendLog(BAD, "no known USB device found");
	return UNKNOWN;
}

//////////////////////////////////////////////////////////////////////////////
bool 
CardDetection::isUSBSerial(bool checkForPcmciaLink)
{ 
	// do we have any usbserial ports at all?
	// and if so: how many?
	int myUSBSerialPort;
	int myFirstUSBSerial   = -1;
	int myHighestUSBSerial = -1;
	int myNumberOfUSBSerialPorts = 0;
	for (myUSBSerialPort=0; myUSBSerialPort<32; myUSBSerialPort++)
	{
		QString myttyUSBx = "/sys/class/tty/ttyUSB" + QString::number(myUSBSerialPort);

		// we have to check for a lot of devices - due to insert/remove/insert/remove issues
		if (!FileStuff::doesFileExist(myttyUSBx))
			continue;

		// a USB serial port was found!
		myNumberOfUSBSerialPorts++;
		DEBUG4("Found serial port number %d at location '%s'\n", myNumberOfUSBSerialPorts, myttyUSBx.ascii());

		// check if this USB serial port is mounted on a PCMCIA card
		if (checkForPcmciaLink)
		{
			QFileInfo myFileInfo (myttyUSBx+"/device");
			QString myUSBdevLink = myFileInfo.readLink();
			if ( myUSBdevLink.contains(thePCMCIASocketDeviceLink) )
			{
				if (myFirstUSBSerial < 0 )
					myFirstUSBSerial = myUSBSerialPort;
				myHighestUSBSerial = myUSBSerialPort;
				DEBUG4("  Port %d resides on a PCMCIA card\n", myNumberOfUSBSerialPorts);
			}
			else
				DEBUG4("  Port %d apparently does not come from a PCMCIA card\n", myNumberOfUSBSerialPorts);
		}
		else
		{
			if (myFirstUSBSerial < 0 )
				myFirstUSBSerial = myUSBSerialPort;
			myHighestUSBSerial = myUSBSerialPort;
		}
		
	}
	if (myNumberOfUSBSerialPorts == 0)
		return false;
	QString myMessage;
	myMessage = "There are " + myMessage.setNum(myNumberOfUSBSerialPorts) + " USB Serial ports available"; 
	sendLog(INFO, myMessage);

	// do we have any usb ports that qualify?
	if (myHighestUSBSerial < 0)
		return false;
	if (checkForPcmciaLink)
		sendLog(GOOD, "some USB ports are from a PCMCIA module");

	// this is a hack to work around the 4-port versions. The 4th port has different
	// line endings :-(
	if (myHighestUSBSerial > myFirstUSBSerial + 2)
	{
		sendLog(GOOD, "4-port version of USBSerial PCMCIA card found - using 3rd port");
		myHighestUSBSerial = myFirstUSBSerial + 2;
	}

	theATSerialPortName = "/dev/ttyUSB" + QString::number(myHighestUSBSerial);
	thePPPSerialPortName = "/dev/ttyUSB" + QString::number(myFirstUSBSerial);

	// probing of this port will be done later...
	return true;
}


//////////////////////////////////////////////////////////////////////////////
bool 
CardDetection::isNozomi(void)
{ 
	
	if (!FileStuff::doesFileContainString("/proc/modules", QString("nozomi")))
	{
		sendLog(KILLING, "No kernel module for nozomi-based card loaded");
		return false;
	}
	if (FileStuff::doesFileContainString("/proc/devices", QString("nozomi")))
	{
		theATSerialPortName  = "/dev/nozomi2";
		thePPPSerialPortName = "/dev/nozomi0";
		sendLog(GOOD, "found /dev/nozomi* - the 'Holtzman' version of Nozomi drivers");
	}
	else
	{
		if (FileStuff::doesFileContainString("/proc/devices", QString("noz")))
		{
			theATSerialPortName = "/dev/noz2";
			thePPPSerialPortName = "/dev/noz0";
			sendLog(GOOD, "found /dev/noz* - the 'Hardwick' version of Nozomi drivers.");
		}
		else
		{
			sendLog(BAD, "No kernel device nozomi found");
			return false;
		}
	}

	if (!FileStuff::doesFileExist(theATSerialPortName))
	{
		sendLog(KILLING, theATSerialPortName + " device entry was not found");
		return false;
	}

	if (!FileStuff::doesFileExist(thePPPSerialPortName))
	{
		sendLog(KILLING, thePPPSerialPortName + " device entry was not found");
		return false;
	}

	sendLog(GOOD, "Found the device entry for a nozomi-based card");
	return true;
}


//////////////////////////////////////////////////////////////////////////////
void
CardDetection::sendLog(LogQuality isGood, const QString& aMessage)
{
	// FIXME: for now, this is just a print to stdout.
	// in the future, this should be calling back a registered function that
	// either dumps to stdout, a log file or to a text view window

	QString myQuality;
	switch (isGood)
	{
	case GOOD:	
		myQuality="GOOD";
		DEBUG2("GOOD: '%s'\n", aMessage.ascii());
		break;
	case BAD:	
		myQuality="BAD";
		DEBUG2("BAD: '%s'\n", aMessage.ascii());
		break;
	case KILLING:	
		myQuality="KILLING";
		DEBUG1("KILLING: '%s'\n", aMessage.ascii());
		break;
	case INFO:	
		myQuality="INFO";
		DEBUG3("INFO: '%s'\n", aMessage.ascii());
		break;
	}
}
