/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DETECTCARD_CLASS
#define DETECTCARD_CLASS

#include "Device.h"


enum LogQuality
{
	GOOD,		// use GOOD if this is a positive ID of a certain card
	BAD,		// use BAD if an optional piece of detection failed, detection might not succeed but can continue
	KILLING,	// use KILLING if a REQUIRED piece of detection cannot be done
	INFO		// use INFO to state that something's present that can help detection
};	

class DetectLogCallbackInterface
{
public:
	virtual ~DetectLogCallbackInterface() {};
	virtual void sendLog(LogQuality isGood, const QString& aMessage);
};

/// holds the modem query plus its result 
class CardDetection
{
public:
	CardDetection(void) : theCallback(NULL),
			      theVendorID(CardDetection::UNKNOWN),
			      theDevicePtr(NULL)	{};

	virtual ~CardDetection() {};

//	void registerLogCallbackInterface(DetectLogCallbackInterface* anInterface);

public:

	// note, this list is a mix of PCI VendorIDs and USB VendorIDs
	enum VendorIDs
	{
		UNKNOWN = 0,
		USER_SPECIFIED,
		UNNAMED_SINGLE_SERIAL,
		UNNAMED_MULTI_SERIAL,
		OPTION = 0x1931,
		OPTION_USB = 0x0af0,
		OPTION_USBSTORAGE = 0x05c6,		// only returned for non-switched ICONs/cards
		FOURGSYSTEMS_USB = 0x11f5,				// USB-ID
		FOURGSYSTEMS_USBSTORAGE = 0x1955,	// only returned for non-switched boxes/cards
		OPTI   = 0x1045,
		KYOCERA = 0x0c88,	// USB-ID
		BROADCOM = 0x14e4,
		NEC	= 0x1033,		// NEC usb2serial is base of Option GT and Psitek/Huawei cards
		PSITEK	= 0x12d1,
		SIERRAWIRELESS = 0x0192,		// this is not a PCI id...
		NOVATEL = 0x1410,		// USB-ID
		NOT_SUPPORTED = 0xFFFF
	};

	const QString& getATSerialPortName() const	{ return theATSerialPortName; };
	const QString& getPPPSerialPortName() const	{ return thePPPSerialPortName;};
	void  setSerialPortNames(const QString& anATSerialPortName, const QString& aPPPSerialPortName);
	
	VendorIDs 	getVendorID()		{ return theVendorID; };
//	unsigned long	getDeviceID()		{ return theDeviceID; };


	/** this member will attempt to autodetect the device and serial ports it uses
	 *  @returns true if successful
	 */ 
	bool detectPorts(void);
	
	/** this member attempts to create the device that was detected
	 *  @returns true if successful
	 */
	bool createDevice(void);
	
	Device* getDevicePtr(void)		{ return theDevicePtr;};

private:
	bool is26kernel(void);
	bool isPCMCIA(void);
	
	/** scan the PCI bus for known PCMCIA devices
	 *  @returns the VendorID if found
	 */
	VendorIDs doPCMCIA_PCIDetection(void);
	
	/** scans sysfs for known USB devices, like the Option ICON and such
	 *  @returns the VendorID if found
	 */	
	VendorIDs doUSBDetection(void);
	
	/** scans for known ACM or RFCOMM phone modems
	 *  @returns the VendorID if found
	 */	
	VendorIDs doPhoneModemDetection(void);
	
	/** scans for USB serial connections 
	 *  @returns the VendorID if found
	 */	
	VendorIDs doRawUSBSerialDetection(void);

	/** checks if a (series of) usbserial port(s) exist.
	 *  if so: it will setup the second or third one for control
	 *  and the first one for PPP.
	 *  @param    checkForPcmciaLink   if true only usbserial hooked up to 
	 *            pcmcia are considered
	 *  @return   true if usbserial ports of the right type were found
	 */
	bool isUSBSerial(bool checkForPcmciaLink = true);
	bool isNozomi(void);

	/** checks if a CS-serial port is active. 
	 *  @return   the vendor ID if a known port, or UNKNOWN if unknown card
	 */
	VendorIDs doCSSerialDetection(void);

	void sendLog(LogQuality isGood, const QString& aMessage);



private:
	DetectLogCallbackInterface*	theCallback;

	QString				theATSerialPortName;
	QString				thePPPSerialPortName;
	
	QString				thePCMCIASocketsDirectory;
	QString				thePCMCIASocketDeviceLink;

	enum VendorIDs			theVendorID;
	enum Device::CardType  theDeviceID;

	Device*				theDevicePtr;

};


#endif // defined DETECTCARD_CLASS
