/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "Query.h"
#include "SerialPort.h"
#include "umtsmon_global.h"

#include <assert.h>


// make sure that upon initialisation, theSerialPortPtr is NULL
static SerialPort* theSerialPortPtr = NULL;
static QMutex	   theQueryMutex;


void Query::setSerial(SerialPort* aSerialPortPtr)
{
    theSerialPortPtr = aSerialPortPtr;
}

	
// FIXME: handy for debugging - to be removed
void printchar(int ch)
{
	switch (ch)
	{
		case '\r':
			DEBUG4("\\r");
			break;
		case '\n':
			DEBUG4("\\n\n");
			break;
		default:
			DEBUG4("%c",ch);
	}
}




Query::~Query()
{
	if (hasLock)
	{
		DEBUG5("Query instance %p abandoned MUTEX\n", this);
		theQueryMutex.unlock();
		hasLock=false;
	}
}




Query::ReturnCode Query::receiveMore()
{
	// as we know the Query has already run, let's clean out the string list first
	theQueryString="";
	isFirstRun = false;
	theAnswer.clear();
	return realRun();
}


Query::ReturnCode Query::runUntilDone()
{
	ReturnCode myResult = run();
	int i=0;
	while (myResult == Query::UNKNOWN_ERROR)
	{
		DEBUG5("Query::runUntilDone needs more time...\n");
		// sleep for 100 ms
		struct timespec sleep_time;
		sleep_time.tv_sec=0;
		sleep_time.tv_nsec=100*1000*1000; 
		nanosleep(&sleep_time,NULL);
		// and retry
		myResult = receiveMore();
		// and check for larger timeouts
		if (i++ > 300)
		{
			DEBUG1("Query::runUntilDone needed more than 30 seconds. Bailing out.\n");
			break;
		}
	}
	
	return myResult;
}

Query::ReturnCode Query::run()
{
	// safety net
	assert(theSerialPortPtr != NULL);
	if (theSerialPortPtr == NULL)
	{
		return NO_DEVICE;
	}

	// more safety net: lock the mutex
	assert(hasLock == false);
	if (theQueryMutex.tryLock()==FALSE)
	{
		hasLock = false;
		DEBUG5("Query instance %p trylock FAILED\n", this);
		return NO_LOCK;
	}
	hasLock=true;
	DEBUG5("Query instance %p acquired MUTEX\n", this);

	// ***** first make sure there's no residual info on the line
	char myByte;
	while ( (myByte=theSerialPortPtr->getOneByte()) > 0)
	{
		DEBUG1("Query: removing excess byte '%c' %d\n", isprint(myByte)?myByte:'.', myByte);
	}

	return realRun();
}

Query::ReturnCode Query::realRun()
{
	// ***** send our message
	if (isFirstRun == true)
	{
		if (theSerialPortPtr->writeDev("\r"+theQueryString+"\r") 
			== false)
		{
			return NO_DEVICE;
		}
	}

	// ***** receive the response
	receiveResponse();

	int i=0;
	for ( QStringList::Iterator myIt = theAnswer.begin(); 
               myIt != theAnswer.end(); ++myIt ) 
	{
		i=i+1;
        DEBUG4("answer %d:'%s'\n", i, (*myIt).stripWhiteSpace().ascii());
	}

	// ***** interpret the response

	if (isFirstRun == true)
	{	
		if (checkQuery() == false)
			return UNKNOWN_ERROR;
	}

	return interpretResult();
}

// read out the data from the serial port
void Query::receiveResponse()
{
	assert(theSerialPortPtr != NULL);

	int ch;
	QString myCompleteAnswer("");
	while( (ch=theSerialPortPtr->getOneByte()) >0)
	{
		printchar(ch);
		myCompleteAnswer += ch;
	}

	theAnswer = QStringList::split("\r\n",
			               myCompleteAnswer);
}

// attempt to interpret the last line (OK/ERROR/etc)
Query::ReturnCode Query::interpretResult(void)
{
	ReturnCode myReturnCode = UNKNOWN_ERROR;

	// let's do the plain, simple answers first
	if (theAnswer.last() == "OK")
		myReturnCode = OK;
	if (theAnswer.last() == "ERROR")
		myReturnCode = ERROR;
	if (theAnswer.last() == "NO CARRIER")
		myReturnCode = NO_CARRIER;
	
	if (myReturnCode != UNKNOWN_ERROR)
	{
		DEBUG5("because known statement #%d was detected, removing obvious answer '%s'\n",
				myReturnCode, theAnswer.last().ascii());	
		theAnswer.pop_back();
	}

	// if the answer however contained a specific error
	// message like +CME ERROR: SIM PIN, let's return
	// error as well - but don't pop the error
	if (theAnswer.last().contains("ERROR",true))
		myReturnCode = ERROR;

	return myReturnCode;
}

// check if first line equals the Query, and remove it if so.
bool Query::checkQuery(void)
{
	if ( theQueryString == (*(theAnswer.begin())).stripWhiteSpace() )
	{
		DEBUG5("checkQuery: removing echo up front\n");
		theAnswer.pop_front();
		return true;
	}
	return false;
}

bool Query::sendMore(const QString& aString)
{
	// safety net
	assert(theSerialPortPtr != NULL);
	if (theSerialPortPtr == NULL)
	{
		return false;
	}

	// ***** send our message
	return theSerialPortPtr->writeDev(aString+"\r");
}


QStringList& Query::getAnswer()
{
	// if this assert kicks in, it probably means you ran getAnswer() before run()
	// OR
	// the Query hasn't completed yet - you might need to run receiveMore() 
	// a few times until the answer gets in...
	DEBUG5("Query::getAnswer returns %zd answer(s)\n", theAnswer.count() ); 
	assert (!theAnswer.isEmpty());
	return theAnswer; 
}
