/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "umtsmon_global.h"

#include "mainwindow.h"
#include "DetectCard.h"
#include "SIMHandler.h"
#include "Runner.h"
#include "Popup.h"

#include "HelperList.h"

#include <qapplication.h>
#include <qmessagebox.h>
#include <assert.h>
#include <signal.h>

// forward declarations
void popupAbortMessageBox(int);
void printAbortMessage(int);


static const int MAXLONGCOMMANDSIZE  = 12;
static const int MAXSHORTCOMMANDSIZE =  4;

struct s_args
{
	char theFullCommand [MAXLONGCOMMANDSIZE];
	char theShortCommand[MAXSHORTCOMMANDSIZE];
	bool needsArgument;
	bool (*theFunctionPtr)(char* anArgument);
};


static bool help( __attribute__ ((unused)) char* anArgument)
{
	printf(APPNAME " " APPRELEASE "\n\nhelp text\n\n");
	printf(" --help             gives this help text\n");
	printf(" -h                 gives this help text\n");
	printf(" --serial <device>  will prevent autodetection of PCMCIA cards and use <device>\n");
	printf(" -s <device>        will prevent autodetection of PCMCIA cards and use <device>\n");
	printf(" --verbose [level]  set verbosity, 1=little (default), %d=all\n", MAX_VERBOSITY);
	printf(" -v [level]         set verbosity\n");
	printf("\n"); 
	exit(1);
}


static bool needsAutoDetect = true;
static QString theUserSerialPortName;

static bool setserial(char* anArgument)
{
	DEBUG5("setserial: '%s'\n", anArgument);

	needsAutoDetect = false;
	theUserSerialPortName = anArgument;
	return true;
}

// oops global variable declared here...
int  theVerbosity = 1;

static bool setVerbosity(char* anArgument)
{
	DEBUG5("setVerbosity: '%s'\n", anArgument);
	
	// the argument should be a number. Let's parse.
	int myNumber = atoi(anArgument);
	if (myNumber < 1 || myNumber > MAX_VERBOSITY)
	{
		printf("ERROR: verbosity argument should be between 1 and 5\n");
		return false;
	}
	theVerbosity = myNumber;
	DEBUG2("set verbosity level to %d\n", theVerbosity);
	
	return true;
}

static struct s_args theArgsTable[] =
{ 
	{ "--help", "-h", false, help, },
	{ "--serial", "-s", true, setserial, },
	{ "--verbosity", "-v", true, setVerbosity, }, 
//	keep this one last
	{ "\0", "\0", false, NULL, },
};

/** compare strings. 
 *  @returns 		If 0, no match, if positive nonzero: the character at this index is extra
 */
static int compareStrings(const char* anArgument, const char* aCommand)
{
	int i=0;
	while(true)
	{
		// we're at the end of the command - a match, maybe with info left...
		if (aCommand[i] == 0)
			return i;
		// the argument is shorter than the command - cannot be a match :-(
		if (anArgument[i] == 0)
			break;
		if (anArgument[i] != aCommand[i])
			break;
		i++;
	}

	// if we get here, it was no match :-(			
	return 0;
}


int main( int argc, char **argv )
{
	
	QApplication a( argc, argv );

	puts( APPNAME " version " APPRELEASE " ." );

	// perform command line parsing
	// skip argv[0] because we know what's in there
	for ( int i = 1; i < a.argc(); i++ )
	{
		int  myMatchPoint = 0;
				
		char* myArgvPtr = a.argv()[i];
		struct s_args* myTEntry;

		for (int j = 0; theArgsTable[j].theFullCommand[0]!='\0'; j++)
		{
			myTEntry = &(theArgsTable[j]);

			myMatchPoint = compareStrings(myArgvPtr, myTEntry->theFullCommand);
			if (myMatchPoint)
				break;
			myMatchPoint = compareStrings(myArgvPtr, myTEntry->theShortCommand);
			if (myMatchPoint)
				break;
		} // end of the 'j' loop

		if (myMatchPoint)
		{
			char* anArgument = NULL;

			// there are several cases:
			//  1)  -s 3
			//  2)  -s3
			//  3)  -s=3
			//  4)  --serial=3
			//  5)  --serial 3
			//  6)  --help     (no arguments)
			//  7)  -h         (no arguments)

			// cases 1,2,3,4,5:
			if (myTEntry->needsArgument)
			{
				// is there extra data in this argument (i.e. cases 2,3,4)
				if (myArgvPtr[myMatchPoint] != 0)
				{
					// is it a '=' (case3,4) - then advance one character
					if (myArgvPtr[myMatchPoint] == '=')
						myMatchPoint++;
					anArgument = &myArgvPtr[myMatchPoint];
				}
				else
				{
					// case 1,5:

					// is there a next argument to parse?
					if ( (i+1) < a.argc())
					{
						anArgument = a.argv()[i+1];
						i++;
					}
					else
						anArgument = NULL;
				}
			}
			
			
			if (myTEntry->needsArgument==true && anArgument==NULL)
			{
				printf("parsing error: missing argument to '%s'!\n", myArgvPtr);
				exit(3);
			}
			if (myTEntry->theFunctionPtr(anArgument) == false)
			{
				printf("...\nExiting gracefully\n\n");
				help(NULL);
				// this should never be reached:
				exit(1);
			}
		}
		else
		{
			printf("ERROR: unknown argument '%s'\n", myArgvPtr);
			exit(2);
		}
	} // end of the 'i' loop

	// If a user who doesn't run umtsmon from the command line hits an assert,
	// the program stops without a visible UI message.
	// let's fix that here: if no verbosity is set, register a SIGABRT handler to pop up a dialog
	if (theVerbosity <= 1)
	{
		printf("installing GUI SIGABRT handler\n");
		signal(SIGABRT, popupAbortMessageBox);
	}
	else
	{
		// also register a handler explaining what to do 
		// if the users enabled lots of verbosity 
		if (theVerbosity >= 5)
		{
			printf("installing text SIGABRT handler\n");
			signal(SIGABRT, printAbortMessage);
		}		
	}

	// if still needed, perform autodetection on 3G PCMCIA cards
	CardDetection myDetection;
	if (!needsAutoDetect)
	{
		myDetection.setSerialPortNames(theUserSerialPortName, theUserSerialPortName);
	}
	else
	{
		if (myDetection.detectPorts()==false)
		{
			if (theVerbosity!=5)
				Popup::Critical("Device detection not successful.\n"
			 				"Please re-run on a command line and check the output:\n"
			 				APPNAME " -v5");
			else
				Popup::Critical("Device detection not successful.");
		}
	}

	// at this point, we know where to expect the serial port(s)
	// let's do some checks on read/write access now - for the ports
	// and also suid checks on the helper applications we have
	HelperList myList1;
	myList1.setSerialPorts(myDetection.getATSerialPortName(), myDetection.getPPPSerialPortName());
	if (myList1.testAndRun() == HelperList::QUIT)
		exit(0);
	
	// check for rootness - useful for the logs if users do odd things
	if (Runner::amIRoot())
		DEBUG1(APPNAME " is running as root!!!\n");

	// OK, we have a suicidal user or everything is fine...
	// let's create the device
	if (myDetection.createDevice()==false)
	{
		if (theVerbosity!=5)
			Popup::Critical("Device creation not successful.\n"
		 				"Please re-run on a command line and check the output:\n"
		 				APPNAME " -v5");
		else
			Popup::Critical("Device creation not successful.");
	}

	{ // scope the SIMHandler because it no longer needs to exist after the askForPIN
	// Everthink fine arount the sim? SIMHandler will check it...
	SIMHandler mySIMHandler;
	mySIMHandler.setDeviceInfo(myDetection.getDevicePtr());
	mySIMHandler.askForPIN(); 
	}

	////////////////////////////////////
	// and start the application
	// do not forget to pass on some initialised classes
	mainwindow w;
	a.setMainWidget( &w );
	w.setDevicePtr(myDetection.getDevicePtr());
	w.show();
	return a.exec();
}





void popupAbortMessageBox(__attribute__ ((unused)) int aSignalNumber)
{
	QMessageBox::critical (NULL, 
			APPNAME " assert error",
			APPNAME " ran into a problem and unfortunately had to close.<br>"
				"It is highly likely that if you run " APPNAME "from a command line, like:\n"
				APPNAME " -v5\n"
				"you will get information on what went wrong, including"
				" information on how to help us solve it.\n"
				"We thank you in advance for helping us improve " APPNAME "!",
			QMessageBox::Ok |  QMessageBox::Default, 
			QMessageBox::NoButton, 
			QMessageBox::NoButton);
	// do not call exit here - or the signal will not be handled correctly
}

void printAbortMessage(__attribute__ ((unused)) int aSignalNumber)
{
	printf("\n\n  *** " APPNAME " version " APPRELEASE " closed due to an"
	           " unrecoverable program error.\n"
	       "  *** Please e-mail all of the above output to"
	           " umtsmon-develop@lists.sourceforge.net,\n"
	       "  *** Or go to the " APPNAME " website and file a bug.\n\n\n"); 
	// do not call exit here - or the signal will not be handled correctly
}
