/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006, 2007  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "umtsmon_global.h"
#include "NetworkChanger.h"
#include "Query.h"
#include "ConnectionInfo.h"

#include <assert.h>

NetworkChanger::~NetworkChanger() 
{
	// to make sure that no Query exists anymore 
	// otherwise will have locking issues elsewhere
	if (theQueryPtr != NULL)
	{
		delete theQueryPtr;
		theQueryPtr = NULL;
	}
	
	while (!theCNlist.empty())
	{
		delete theCNlist.back();
		theCNlist.pop_back();
	}
};

bool 
NetworkChanger::startSearching(void)
{
	assert(theQueryPtr == NULL);
	theQueryPtr = new Query("AT+COPS=?");

	Query::ReturnCode myResult = theQueryPtr->run();
	if (myResult == Query::ERROR)
		return false;
	return true;
}


bool 
NetworkChanger::isSearchCompleted(void)
{
	assert(theQueryPtr != NULL);
	Query::ReturnCode myCode = theQueryPtr->receiveMore();
	if (myCode != Query::OK)
	{
		return false;
	}

	// if there is more than one answer, we will have to iterate through them...
	for (QStringList::Iterator myIterator = theQueryPtr->getAnswer().begin();
		myIterator !=  theQueryPtr->getAnswer().end(); myIterator++)
	{
		QString myAnswer = *myIterator;
		parseLine(myAnswer);
	}

	delete theQueryPtr;
	theQueryPtr = NULL;
	return true;
}


void 
NetworkChanger::parseOperator(const QString& aFullTelcoString)
{
/*
DEBUG5("NetworkChanger::parseLine(%s)\n", aFullTelcoString.ascii());
DEBUG5("  allow: '%s'\n", aFullTelcoString.section(",",0,0).ascii());
DEBUG5("  full:  '%s'\n", aFullTelcoString.section(",",1,1).ascii());
DEBUG5("  short: '%s'\n", aFullTelcoString.section(",",2,2).ascii());
DEBUG5("  id:    '%s'\n", aFullTelcoString.section(",",3,3).ascii());
DEBUG5("  type:  '%s'\n", aFullTelcoString.section(",",4,4).ascii());
*/	
	CurrentNetworkItem* myCNIPtr = new CurrentNetworkItem;

	switch (aFullTelcoString.section(",",0,0).toInt())
	{
	case 3: // forbidden
		myCNIPtr->theIconType = FORBIDDEN;
		break;
	case 1: // available
		myCNIPtr->theIconType = AVAILABLE;
		break;
	case 2:	// current
		myCNIPtr->theIconType = CURRENT;
		break;
	case 0:	// unknown -> denotes the end of the list
		return;
	}

	myCNIPtr->theNetworkID = 
			aFullTelcoString.section(",",3,3)
			.section("\"",1,1).toLong();

	QString myRadiotype =
		ConnectionInfo::getRadiotypeString(
			aFullTelcoString.section(",",4,4).toInt());

	// do we have that network ID already?
	CurrentNetworkItem* myDuplicateNetworkID = findNetworkID(myCNIPtr->theNetworkID);
	if (myDuplicateNetworkID != NULL)
	{
		// add the radio type to the name
		myDuplicateNetworkID->theNetworkName += "+" + myRadiotype;

		// if this one is current, update entry
		// FIXME: is it possible within one provider that one network is forbidden
		//        whilst the other is current???
		if (myCNIPtr->theIconType == CURRENT)
			myDuplicateNetworkID->theIconType = CURRENT;
	}		
	else
	{
		myCNIPtr->theNetworkName=
			 aFullTelcoString.section(",",1,1) 
				+ "  -  " + myRadiotype;
		theCNlist.push_back(myCNIPtr);
	}
}

void 
NetworkChanger::parseLine(const QString& anAnswer)
{
	QStringList  myList = QStringList::split("),", anAnswer);

	// in case of multiple answers on one line (e.g. Option)
	// we will iterate over all the answers here
	int i=0;
	for ( QStringList::Iterator myIt = myList.begin(); 
		myIt != myList.end(); ++myIt ) 
	{
		i=i+1;
		// kill everything up to the first '('
		QString myFullTelcoString = (*myIt).section("(",1,1);
		parseOperator(myFullTelcoString);
	}
}


NetworkChanger::CurrentNetworkItem*
NetworkChanger::findNetworkID(long aNetworkID)
{
	for (CNlist::iterator i=theCNlist.begin(); 
			i != theCNlist.end(); i++)
	{
		if ((*i)->theNetworkID == aNetworkID)
		{
			return *i;
		}
	}
	return NULL;
}



/**************************************************************************** 
 * BASICALLY THERE IS A HARD LINE BETWEEN THE ABOVE AND THE BELOW 
 *     - Above is ask for a list of networks
 *     - Below is switching to one of the network IDs
 */


bool 
NetworkChanger::startSwitchToNetworkID(long aNetworkID)
{
DEBUG5("NetworkChanger::startSwitchToNetworkID\n");

	CurrentNetworkItem* myID = findNetworkID(aNetworkID);
	if(myID == NULL)
	{
		// something's really wrong here
		assert(false);
		return false;
	}

	switch(myID->theIconType)
	{
	case FORBIDDEN:
		return false;
	case CURRENT:
// FIXME: you shouldn't need to switch if your network is current
//		return true;
	case AVAILABLE:	
		{
		QString myTemp;
		QString myAT = "AT+COPS=1,2,\"" + myTemp.setNum(aNetworkID) + "\"";
	
		assert(theQueryPtr == NULL);
		theQueryPtr = new Query(myAT);
		// because the user has already waited 40 seconds to get this far,
		// I want to try very hard to make this proceed - so, if there's something
		// blocking the Query, we retry...
		for (int i=0; i<3; i++)
		{
			Query::ReturnCode myResult = theQueryPtr->run();
			if (myResult == Query::ERROR)
				return false;
			if (myResult != Query::NO_LOCK)
				return true;
			// if we get here, we couldn't get a lock
			// so sleep for a second and retry
			sleep(1);
		} // end-of-while
		// if we get here, starting the Query timed out 3 times...
		break;
		} // end-of-case
	} // end-of-switch

	delete theQueryPtr;
	theQueryPtr = NULL;
	return false;
}


NetworkChanger::SwitchResults 
NetworkChanger::isSwitchCompleted(void)
{
	assert(theQueryPtr != NULL);
	SwitchResults myResult = NOT_READY_YET;
	switch (theQueryPtr->receiveMore())
	{
	case Query::OK:
		// we're done!!!
		myResult = OK;
		break;
	case Query::ERROR:
	case Query::NO_CARRIER:
	case Query::NO_DEVICE:
	case Query::NO_LOCK:
		// I/O troubles or real errors
		myResult = ERROR;
		break;
	case Query::UNKNOWN_ERROR:
		// NOT FINISHED YET
		return NOT_READY_YET;	
	}
	
	delete theQueryPtr;
	theQueryPtr = NULL;
	return myResult;
}

